import socket
import threading
import time

UDP_IP = "255.255.255.255"
UDP_PORT = 5005
LISTEN_PORT = 5006

def broadcast_presence():
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)

    while True:
        message = b"ROBOBIN_PRESENT"
        sock.sendto(message, (UDP_IP, UDP_PORT))
        print("Broadcasting: {}".format(message.decode()))
        time.sleep(5)

def handle_client_connection(client_socket):
    try:
        while True:
            request = client_socket.recv(1024)
            if not request:
                print("No request received, closing connection.")
                break  # Connection closed by the client
            message = request.decode()
            print("Received from client: {}".format(message))

            if message == "PING":
                print("Received PING from client.")  
                response = b"PONG"  
                client_socket.sendall(response)

    except ConnectionResetError:
        print("Client connection was forcibly closed.")
    except Exception as e:
        print(f"An error occurred while handling the client connection: {e}")
    finally:
        client_socket.close()
        print("Client disconnected.")

def listen_for_connections():
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.bind(('', LISTEN_PORT))

    while True:
        try:
            data, addr = sock.recvfrom(1024)
            if data.decode() == "CONNECT":
                print("Received connection request from {}".format(addr))

                # Create a TCP socket to accept connections
                with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as tcp_socket:
                    tcp_socket.bind(('', 5006))  # Listen on the fixed TCP port
                    tcp_socket.listen(1)
                    print("Listening for TCP connection...")

                    client_socket, client_addr = tcp_socket.accept()
                    print("Client connected from {}".format(client_addr))

                    # Spawn a new thread for handling the client connection
                    threading.Thread(target=handle_client_connection, args=(client_socket,)).start()
        except Exception as e:
            print(f"An error occurred while listening for connections: {e}")

# Start the broadcasting and listening in separate threads
broadcast_thread = threading.Thread(target=broadcast_presence)
listen_thread = threading.Thread(target=listen_for_connections)

broadcast_thread.start()
listen_thread.start()

broadcast_thread.join()
listen_thread.join()
